package demo;

import java.io.InputStream;

import java.util.Date;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.activation.DataSource;

import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;


/**
 * Send email messages
 *
 * Setup:
 *
 * Copy this source to an IFS file named Mailer.java. I created Mailer in a 
 * demo class, which corresponds to the package name found at the top of this 
 * program. To use another directory like com/mycompany change the package name 
 * to com.mycompany. I created the Mailer.java class file in a /java/demo 
 * directory.
 *
 * You will also need the mail.jar and activation.jar files on your system. 
 * You can those from Sun's http://java.sun.com/products/javamail/ website to 
 * download the JavaMail framework, which contains mail.jar. Follow the JAF 
 * framework link on that page and download the JAF framework, which contains 
 * the activation.jar. I copied these jar files to /java/javamail and
 * /java/jaf directories.
 *
 * Once you have the Mailer.java file created and mail.jar and activation.jar 
 * on your iSeries system, start qshell and use javac to compile Mailer.java 
 * into a class file. Use the following commands on one line each:
 *
 * cd /java
 * javac -classpath '.:/java/javamail/mail.jar:/java/jaf/activation.jar' 
 *       demo/Mailer.java
 *
 * Usage:
 *
 * Set a mail.smtp.host system property that points to your SMTP server. To 
 * use password authentication, set mail.user and mail.password. You can set 
 * these properties in a file called Mailer.properties in the same directory 
 * as the Mailer class or you can pass these properties as system properties 
 * like:
 *
 * java -Dmail.smtp.host=mymail.mycompany.com -Dmail.user=myUser 
 *      -Dmail.password=myPassword mypackage.MyClass
 *
 * @author  David Morris
 */
public class Mailer {
    public static final String MAILER_PROPERTIES = "Mailer.properties";
    static Properties properties;

    static {
        // If properties file found, use it for user/password
        InputStream is = Thread.currentThread().getContextClassLoader()
                               .getResourceAsStream(Mailer.class.getPackage().getName()
                + "/" + MAILER_PROPERTIES);

        // Load Properties File
        if (is != null) {
            try {
                properties = new Properties();
                properties.load(is);
            }
            catch (Exception e) {
                System.err.println("Error " + "\"" + e.getLocalizedMessage()
                    + "\"" + " loading AS/400 Connection Properties.");
                throw new RuntimeException(e);
            }
        }

        // Assume properties passed as system properties
        else {
            properties = System.getProperties();
        }
    }

    /**
     * All methods are static -- no need to create instance
     */
    private Mailer() {
    }

    /**
     * Retrieve a session with an authenticator using mail.smtp.host, 
     * mail.user, and mail.password.
     * @return session
     */
    private static Session getSession() {
        Authenticator authenticator = null;

        if (properties.getProperty("mail.user") != null) {
            authenticator = new Authenticator() {
                        /**
                         * @see javax.mail.Authenticator#getPasswordAuthentication()
                         */
                        protected PasswordAuthentication getPasswordAuthentication() {
                            return new PasswordAuthentication(properties
                                .getProperty("mail.user"),
                                properties.getProperty("mail.password"));
                        }
                    };
        }

        return Session.getInstance(properties, authenticator);
    }

    /**
     * Send a simple email.
     * @param to        The destination address
     * @param from      The source address
     * @param subject   The subject line for the email
     * @param message   The message for the email
     * @throws MessagingException
     */
    public static void sendSimpleMail(String to, String from, String subject,
        String message)
        throws MessagingException {
        Session session = getSession();

        MimeMessage mimeMessage = new MimeMessage(Session.getDefaultInstance(
                    properties, null));

        InternetAddress iaFrom = new InternetAddress(from);
        mimeMessage.setFrom(iaFrom);

        InternetAddress iaTo = new InternetAddress(to);
        mimeMessage.addRecipient(Message.RecipientType.TO, iaTo);

        mimeMessage.setSubject(subject);
        mimeMessage.setText(message);

        Transport.send(mimeMessage);
    }

    /**
     * Send a multipart email.
     * @param to        The destination address
     * @param from      The source address
     * @param subject   The subject line for the email
     * @param message   The message for the email
     * @param bodyParts The attachments
     * @throws MessagingException
     */
    public static void sendMultiPartMail(String to, String from,
        String subject, String message, DataSource[] bodyParts)
        throws MessagingException {
        MimeMessage mimeMessage = new MimeMessage(getSession());
        InternetAddress[] address = {
            new InternetAddress(to)
        };
        mimeMessage.setRecipients(Message.RecipientType.TO, address);
        mimeMessage.setFrom(new InternetAddress(from));
        mimeMessage.setSubject(subject);
        mimeMessage.setSentDate(new Date());

        MimeBodyPart mbpMessage = new MimeBodyPart();
        mbpMessage.setText(message);
        mbpMessage.setContent(message, "text/html");

        Multipart mp = new MimeMultipart();
        mp.addBodyPart(mbpMessage);

        for (int i = 0; i < bodyParts.length; i++) {
            MimeBodyPart mbp = new MimeBodyPart();
            mbp.setDataHandler(new DataHandler(bodyParts[i]));
            mbp.setFileName(bodyParts[i].getName());
            mp.addBodyPart(mbp);
        }

        mimeMessage.setContent(mp);
        mimeMessage.saveChanges();

        Transport.send(mimeMessage);
    }
}
